/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __network_SignalSender_h__
#define __network_SignalSender_h__

namespace fe
{

/**	Sends signals over a given Socket.  This class is a signal handler
	and simply sends record data over the associated Socket when it handles
	a signal.

	This class is intended to be used in conjuction with SignalReceiver.

	@copydoc SignalSender_info
	*/
class SignalSender :
	virtual public HandlerI,
	virtual public BindI,
	public Initialize<SignalSender>
{
	public:
		SignalSender(int sid, sp<Scope> spScope,
			sp<ChunkSpool> spChunkSpool);
virtual	~SignalSender(void);
		void	initialize(void);

virtual	void	handle(Record &signal);
	/**	As a HandlerI this function is called when the handler is registered
		with a SignalerI.  When this happens a layout description is
		sent to the socket. */
virtual	void	bind(sp<SignalerI> spSignalerI, sp<Layout> spLayout);

		void	setMaxSpoolSize(unsigned int a_size) {m_maxspoolsize=a_size;}

	private:
		sp<RecordGroup>	m_spRG;
		sp<data::StreamI>	m_spStream;
		sp<Scope>			m_spScope;
		sp<ChunkSpool>		m_spChunkSpool;
		FE_UWORD				m_sid;
		Accessor<int>		m_aSID;
		unsigned int		m_maxspoolsize;
};

} /* namespace */

#endif /* __network_SignalSender_h__ */

