/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "platform.pmh"

namespace fe
{

#if FE_OS==FE_WIN32 || FE_OS==FE_WIN64

BWORD DL_Loader::openDL(String filename,BWORD adaptname)
{
	m_name=filename;

	if(adaptname)
	{
		String longname;
		longname.sPrintf("%s.dll",filename.c_str());
		filename=longname;
	}
	m_handle=LoadLibrary(filename.c_str());

	if(!m_handle)
		return FALSE;

	return TRUE;
}

BWORD DL_Loader::closeDL(void)
{
	return FreeLibrary(m_handle);
}

void *DL_Loader::findDLFunction(String symbol)
{
	return GetProcAddress(m_handle,symbol.c_str());
}

#else
//* Unix

BWORD DL_Loader::openDL(String filename,BWORD adaptname)
{
	m_name=filename;

	if(adaptname)
	{
		String longname;
#if FE_OS==FE_OSX
		longname.sPrintf("lib%s.dylib",filename.c_str());
#else
		longname.sPrintf("lib%s.so",filename.c_str());
#endif
		filename=longname;
	}

	//* RTLD_LAZY don't resolve symbols until you need them
	//* RTLD_NOW resolves symbols immediately or fails
	//* exactly one of RTLD_LAZY and RTLD_NOW must be specified
	//* RTLD_GLOBAL symbols from the DL can be used in upcoming DL's
	//* RTLD_DEEPBIND (new?) chooses local symbols over existing symbols
	//* RTLD_NOLOAD can be used on already loaded DL to change flags
	//* NOTE RTLD_GLOBAL is required for cross-lib dynamic_cast<>
	int dlflags=RTLD_GLOBAL|RTLD_NOW;

	m_handle=dlopen(filename.c_str(),dlflags);
//	feLogDirect("DL_Loader::openDL dlopen %s %p\n",filename.c_str(),m_handle);
	if(!m_handle)
	{
#if FE_CODEGEN<=FE_DEBUG
		feLogDirect("%sDL_Loader::openDL(%s) failed\n      %s\n",
				PrefixLog::prefix().c_str(),filename.c_str(),dlerror());
#endif
		return FALSE;
	}

//	printf("DL_Loader::OpenDL() dlopen succeeded\n");

	return TRUE;
}

BWORD DL_Loader::closeDL(void)
{
//	feLogDirect("DL_Loader::closeDL dlclose %p\n",m_handle);
	if(!m_handle)
		return FALSE;

#if FALSE&& FE_CODEGEN==FE_PROFILE
	feLogDirect("DL_Loader::closeDL ignored for FE_PROFILE (%s)\n",m_name.c_str());
	return TRUE;
#endif

	int fail=dlclose(m_handle);
	if(fail)
	{
		feLogDirect("%sDL_Loader::closeDL failed\n      %s\n",
				PrefixLog::prefix().c_str(),dlerror());
	}
	return !fail;
}

void *DL_Loader::findDLFunction(String symbol)
{
	void* address=dlsym(m_handle,symbol.c_str());
#if FALSE // FE_CODEGEN<=FE_DEBUG
	if(!address)
	{
		feLogDirect("DL_Loader::findDLFunction(%s) failed\n  %s\n",
				symbol.c_str(),dlerror());
	}
#endif
	return address;
}

#endif

} // namespace
